@@ -22,6 +22,7 @@ class Window(kaa.context.Context):
     closed = False
     editmode = None
     cursor = None
+
     def __init__(self, parent, wnd=None, pos=None):
         """Wrap window object wnd. Create new window if wnd was omitted."""
 
@@ -89,17 +90,19 @@ class Window(kaa.context.Context):
     _lasterror = ''
     _skipped = 0
 
-    def do_input(self, nonblocking):
-        """Get an input from curses"""
-
-        self._panel.top()
+    def restore_cursor_pos(self):
         if self.cursor:
-            x, y = self.cursor.screenpos
+            x, y = self.cursor.last_screenpos
             if (y, x) != self._cwnd.getyx():
                 h, w = self._cwnd.getmaxyx()
                 if y < h and x < w and y >= 0 and x >= 0:
                     self._cwnd.move(y, x)
 
+    def do_input(self, nonblocking):
+        """Get an input from curses"""
+
+        self._panel.top()
+
         if nonblocking:
             self._cwnd.timeout(0)
         else:
